/*
 * This file is part of OpenCorsairLink.
 * Copyright (C) 2017,2018  Sean Nelson <audiohacked@gmail.com>

 * OpenCorsairLink is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.

 * OpenCorsairLink is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with OpenCorsairLink.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "device.h"
#include "driver.h"
#include "lowlevel/commanderpro.h"
#include "print.h"
#include "protocol/commanderpro.h"

#include <errno.h>
#include <libusb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

int corsairlink_commanderpro_get_fan_setup_mask(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t* sensor_index )
{
    int rr;
    int sensor;
    //char* sensor;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    msg_debug2( "call started\n" );
    //sensor = printf( sensor_index, 1 );
    //print( ( sensor* ), sensor_index );
    commands[0] = 0x20;
    commands[1] = 0; //sensor_index;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 16 );
    sensor = sensor_index + 1;
    rr = response[sensor];
    //msg_debug2( "Returning %d\n", response[atoi( sensor )] );
    msg_debug2( "Fan %d selected: \n", sensor_index + 1 );
    for ( int ii = 1; ii <= 6; ++ii )
    {
        msg_debug2( "Fan %d: ", ii );

        switch ( response[ii] )
        {
        case 0x00:
            msg_debug2( "Auto/Disconnected\n" );
            break;
        case 0x01:
            msg_debug2( "3-Pin\n" );
            break;
        case 0x02:
            msg_debug2( "4-Pin\n" );
            break;
        case 0x03:
            msg_debug2( "Unknown\n" );
            break;
        default:
            msg_debug2( "Impossible!\n" );
        }

    }

    return rr;
}

int corsairlink_commanderpro_get_fan_speed_rpm(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint16_t* rpm )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x21;
    commands[1] = sensor_index;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 16 );

    *( rpm ) = ( response[1] << 8 ) + response[2];
    for ( int ii = 0; ii <= 16; ++ii )
    {
        msg_debug( "commands : %d\n", commands[ii] );
    }
    for ( int ii = 0; ii <= 16; ++ii )
    {
        msg_debug( "response : %d\n", response[ii] );
    }
    return rr;
}

int corsairlink_commanderpro_get_fan_speed_pwm(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t* pwm )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x22;
    commands[1] = sensor_index;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 16 );

    *( pwm ) = response[1];
    for ( int ii = 0; ii <= 14; ++ii )
    {
        msg_debug( "commands : %d\n", commands[ii] );
    }
    for ( int ii = 0; ii <= 16; ++ii )
    {
        msg_debug( "response : %d\n", response[ii] );
    }
    
    return rr;
}

int corsairlink_commanderpro_set_fan_speed_pwm(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t pwm )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x23;
    commands[1] = sensor_index;
    commands[2] = pwm;
    // commands[4] = sensor_index;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 16 );
    for ( int ii = 0; ii <= 14; ++ii )
    {
        msg_debug( "commands : %d\n", commands[ii] );
    }
    for ( int ii = 0; ii <= 16; ++ii )
    {
        msg_debug( "response : %d\n", response[ii] );
    }
    return rr;
}

int corsairlink_commanderpro_set_fan_mode(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t mode )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x28;
    commands[1] = 0x02;
    commands[2] = sensor_index;
    commands[3] = mode;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 32 );
    for ( int ii = 0; ii <= 14; ++ii )
    {
        msg_debug( "commands : %d\n", commands[ii] );
    }
    for ( int ii = 0; ii <= 32; ++ii )
    {
        msg_debug( "response : %d\n", response[ii] );
    }
    return rr;
}

int corsairlink_commanderpro_get_fan_detect_type(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t* device_id )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x29;
    commands[1] = sensor_index;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 32 );

    return rr;
}

int corsairlink_commanderpro_set_fan_curve( struct corsair_device_info* dev,
                                            struct libusb_device_handle* handle,
                                            uint8_t sensor_index,
                                            uint8_t* device_id )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x25;
    commands[1] = sensor_index;
    commands[2] = 0x00; // 0x00 = CP Temp Probe 1 .... 0x03 = CP Temp Probe 4,
                        // 0xff = External

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 32 );

    return rr;
}

int corsairlink_commanderpro_send_temperature_info(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t temperature )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x26;
    commands[1] = sensor_index;
    commands[2] = temperature >> 8;
    commands[3] = temperature & 0xff;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 32 );

    return rr;
}

int corsairlink_commanderpro_set_fan_connection_mode(
    struct corsair_device_info* dev, struct libusb_device_handle* handle,
    uint8_t sensor_index, uint8_t mode )
{
    int rr;
    uint8_t response[16];
    uint8_t commands[64];
    memset( response, 0, sizeof( response ) );
    memset( commands, 0, sizeof( commands ) );

    commands[0] = 0x28;
    commands[1] = 0x02;
    commands[2] = sensor_index;
    // switch(response[ii])
    // {
    // case 0x00:
    //     msg_debug2("Auto/Disconnected");
    //     break;
    // case 0x01:
    //     msg_debug2("3-Pin");
    //     break;
    // case 0x02:
    //     msg_debug2("4-Pin");
    //     break;
    // case 0x03:
    //     msg_debug2("Unknown");
    //     break;
    // default:
    //     msg_debug2("Impossible!");
    // }
    commands[3] = mode;

    rr = dev->driver->write( handle, dev->write_endpoint, commands, 14 );
    rr = dev->driver->read( handle, dev->read_endpoint, response, 32 );

    return rr;
}
